@@ -4,16 +4,12 @@ package # hide from PAUSE
 use 5.008;
 use strict;
 use warnings;
-use Carp;
+use Carp 'croak';
 use constant PLACEHOLDER => "\x{b1}\x{a4}\x{221e}";
 
 # Common routines and variables exported to all DBO classes.
 # This module automatically exports ALL the methods and variables for use in the other DBO modules.
 
-use subs qw(ouch oops);
-*oops = \&Carp::carp;
-*ouch = \&Carp::croak;
-
 our %Config = (
     AutoReconnect => 0,
     DebugSQL => 0,
@@ -21,21 +17,10 @@ our %Config = (
     RowClass => undef,
     StoreRows => 0,
 );
-our @CARP_NOT;
+our @CARP_NOT = qw(DBIx::DBO DBIx::DBO::Table DBIx::DBO::Query DBIx::DBO::Row);
 our $placeholder = PLACEHOLDER;
 $placeholder = qr/\Q$placeholder/;
 
-sub import {
-    my $caller = caller;
-    push @CARP_NOT, $caller;
-    no strict 'refs';
-    *{$caller.'::Config'} = \%{__PACKAGE__.'::Config'};
-    *{$caller.'::CARP_NOT'} = \@{__PACKAGE__.'::CARP_NOT'};
-    for (qw(oops ouch)) {
-        *{$caller.'::'.$_} = \&{$_};
-    }
-}
-
 sub dbh { $_[0]{DBO}->dbh }
 sub rdbh { $_[0]{DBO}->rdbh }
 
@@ -94,7 +79,7 @@ sub _bind_params_select {
 sub _build_sql_update {
     my $me = shift;
     my $h = shift;
-    ouch 'Update is not valid with a GROUP BY clause' if $me->_build_group($h);
+    croak 'Update is not valid with a GROUP BY clause' if $me->_build_group($h);
     my $sql = 'UPDATE '.$me->_build_from($h);
     $sql .= ' SET '.$me->_build_set($h, @_);
     $sql .= ' WHERE '.$_ if $_ = $me->_build_where($h);
@@ -114,7 +99,7 @@ sub _bind_params_update {
 sub _build_sql_delete {
     my $me = shift;
     my $h = shift;
-    ouch 'Delete is not valid with a GROUP BY clause' if $me->_build_group($h);
+    croak 'Delete is not valid with a GROUP BY clause' if $me->_build_group($h);
     my $sql = 'DELETE FROM '.$me->_build_from($h);
     $sql .= ' WHERE '.$_ if $_ = $me->_build_where($h);
     $sql .= ' ORDER BY '.$_ if $_ = $me->_build_order($h);
@@ -188,14 +173,14 @@ sub _valid_col {
     for my $tbl ($me->tables) {
         return $col if $col->[0] == $tbl;
     }
-    ouch 'Invalid column, the column is from a table not included in this query';
+    croak 'Invalid column, the column is from a table not included in this query';
 }
 
 sub _parse_col {
     my ($me, $col, $_check_aliases) = @_;
     if (ref $col) {
         return $me->_valid_col($col) if UNIVERSAL::isa($col, 'DBIx::DBO::Column');
-        ouch 'Invalid column: '.$col;
+        croak 'Invalid column: '.$col;
     }
     # If $_check_aliases is not defined dont accept an alias
     $me->column($col, $_check_aliases || 0);
@@ -214,7 +199,7 @@ sub _parse_val {
     my $func;
     my $opt;
     if (ref $fld eq 'SCALAR') {
-        ouch 'Invalid '.($c{Check} eq 'Column' ? 'column' : 'field').' reference (scalar ref to undef)'
+        croak 'Invalid '.($c{Check} eq 'Column' ? 'column' : 'field').' reference (scalar ref to undef)'
             unless defined $$fld;
         $func = $$fld;
         $fld = [];
@@ -222,12 +207,12 @@ sub _parse_val {
         $func = $fld->{FUNC} if exists $fld->{FUNC};
         $opt->{AS} = $fld->{AS} if exists $fld->{AS};
         if (exists $fld->{ORDER}) {
-            ouch 'Invalid ORDER, must be ASC or DESC' if $fld->{ORDER} !~ /^(A|DE)SC$/i;
+            croak 'Invalid ORDER, must be ASC or DESC' if $fld->{ORDER} !~ /^(A|DE)SC$/i;
             $opt->{ORDER} = uc $fld->{ORDER};
         }
         $opt->{COLLATE} = $fld->{COLLATE} if exists $fld->{COLLATE};
         if (exists $fld->{COL}) {
-            ouch 'Invalid HASH containing both COL and VAL' if exists $fld->{VAL};
+            croak 'Invalid HASH containing both COL and VAL' if exists $fld->{VAL};
             my @cols = ref $fld->{COL} eq 'ARRAY' ? @{$fld->{COL}} : $fld->{COL};
             $fld = [ map $me->_parse_col($_, $c{Aliases}), @cols ];
         } else {
@@ -242,9 +227,9 @@ sub _parse_val {
     my $with = @$fld;
     if (defined $func) {
         my $need = $me->_substitute_placeholders($func);
-        ouch "The number of params ($with) does not match the number of placeholders ($need)" if $need != $with;
+        croak "The number of params ($with) does not match the number of placeholders ($need)" if $need != $with;
     } elsif ($with != 1 and $c{Check} ne 'Auto') {
-        ouch 'Invalid '.($c{Check} eq 'Column' ? 'column' : 'field')." reference (passed $with params instead of 1)";
+        croak 'Invalid '.($c{Check} eq 'Column' ? 'column' : 'field')." reference (passed $with params instead of 1)";
     }
     return ($fld, $func, $opt);
 }
@@ -272,7 +257,7 @@ sub _build_val {
         } elsif (ref $_ eq 'SCALAR') {
             $$_;
         } else {
-            ouch 'Invalid field: '.$_;
+            croak 'Invalid field: '.$_;
         }
     } @$fld;
     unless (defined $func) {
@@ -281,7 +266,7 @@ sub _build_val {
     }
     # Add one value to @ary to make sure the number of placeholders & values match
     push @ary, 'Error';
-    $func =~ s/$placeholder/shift @ary/eg;
+    $func =~ s/$placeholder/shift @ary/ego;
     # At this point all the values should have been used and @ary must only have 1 item!
     die "Number of placeholders and values don't match @ary!" if @ary != 1;
     return $func.$extra;
@@ -348,7 +333,7 @@ sub _build_where_piece {
 
 # Construct one WHERE expression (simple)
 sub _build_quick_where {
-    ouch 'Wrong number of arguments' if @_ & 1;
+    croak 'Wrong number of arguments' if @_ & 1;
     my ($me, $bind) = splice @_, 0, 2;
     my @where;
     while (my ($col, $val) = splice @_, 0, 2) {
@@ -370,7 +355,7 @@ sub _build_quick_where {
 }
 
 sub _build_set {
-    ouch 'Wrong number of arguments' if @_ & 1;
+    croak 'Wrong number of arguments' if @_ & 1;
     my $me = shift;
     my $h = shift;
     undef @{$h->{Set_Bind}};
@@ -425,27 +410,50 @@ sub _build_limit {
 sub _set_config {
     my $me = shift;
     my ($ref, $opt, $val) = @_;
-    ouch "Invalid value for the 'UseHandle' setting"
+    croak "Invalid value for the 'UseHandle' setting"
         if $opt eq 'UseHandle' and $val and $val ne 'read-only' and $val ne 'read-write';
     my $old = $ref->{$opt};
     $ref->{$opt} = $val;
     return $old;
 }
 
+my %inheritance;
+for my $class (qw(DBIx::DBO DBIx::DBO::Table DBIx::DBO::Query DBIx::DBO::Row DBIx::DBO::Common)) {
+    mro::set_mro($class, 'c3');
+    $inheritance{$class} = {};
+}
 sub _set_dbd_inheritance {
     my $class = shift;
     my $dbd = shift;
     $class =~ s/::DBD::\w+$//;
-    # Inheritance
-    no strict 'refs';
-    unless (@{$class.'::DBD::'.$dbd.'::ISA'}) {
-        my @isa = grep $_->can('_set_dbd_inheritance'), @_ ? @_ : @{$class.'::ISA'};
-        $_->_set_dbd_inheritance($dbd) for @isa;
-        @{$class.'::DBD::'.$dbd.'::ISA'} = ($class, map $_.'::DBD::'.$dbd, @isa);
+
+    my $need_c3_initialize;
+    unless (exists $inheritance{$class}) {
+        mro::set_mro($class, 'c3');
+        $need_c3_initialize = 1 if $] < 5.009_005;
+    }
+    unless (exists $inheritance{$class}{$dbd}) {
+        no strict 'refs';
+        my $dbd_exists = exists ${$class.'::'}{'DBD::'} && exists ${$class.'::DBD::'}{$dbd.'::'};
+        unless ($dbd_exists and @{$class.'::DBD::'.$dbd.'::ISA'}) {
+            my @isa = map $_->_set_dbd_inheritance($dbd), grep $_->isa(__PACKAGE__), @{$class.'::ISA'};
+            unless ($dbd_exists or @isa) {
+                $inheritance{$class}{$dbd} = undef;
+                Class::C3::initialize() if $need_c3_initialize;
+                return wantarray ? () : $class;
+            }
+            @{$class.'::DBD::'.$dbd.'::ISA'} = ($class, @isa);
+        }
+        push @CARP_NOT, $class.'::DBD::'.$dbd;
         mro::set_mro($class.'::DBD::'.$dbd, 'c3');
         Class::C3::initialize() if $] < 5.009_005;
+        $inheritance{$class}{$dbd} = $class.'::DBD::'.$dbd;
     }
-    return $class.'::DBD::'.$dbd;
+    return $inheritance{$class}{$dbd} || (wantarray ? () : $class);
 }
 
+sub _table_class { 'DBIx::DBO::Table' }
+sub _query_class { 'DBIx::DBO::Query' }
+sub _row_class   { 'DBIx::DBO::Row' }
+
 1;
