@@ -3,7 +3,8 @@ package DBIx::DBO::Table;
 use strict;
 use warnings;
 use DBIx::DBO::Common;
-our @ISA;
+use Carp 'croak';
+our @ISA = qw(DBIx::DBO::Common);
 
 use overload '**' => \&column, fallback => 1;
 
@@ -50,20 +51,17 @@ Tables can be specified by their name or an arrayref of schema and table name or
 =cut
 
 sub new {
-    my ($proto, $dbo, $table) = @_;
+    my $proto = shift;
+    UNIVERSAL::isa($_[0], 'DBIx::DBO') or croak 'Invalid DBO Object';
     my $class = ref($proto) || $proto;
-    UNIVERSAL::isa($dbo, 'DBIx::DBO') or ouch 'Invalid DBO Object';
-    (my $schema, $table, my $me) = $dbo->table_info($table) or ouch 'No such table: '.$table;
-    $class = $class->_set_dbd_inheritance($dbo->{dbd});
-    bless { %$me, Schema => $schema, Name => $table, DBO => $dbo, LastInsertID => undef }, $class;
+    $class = $class->_set_dbd_inheritance($_[0]{dbd});
+    $class->_init(@_);
 }
 
-sub _set_dbd_inheritance {
-    my $class = shift;
-    my $dbd = shift;
-    # Let DBIx::DBO::Table secretly inherit from DBIx::DBO::Common
-    @_ = (@ISA, 'DBIx::DBO::Common') if not @_ and $class eq __PACKAGE__;
-    $class->DBIx::DBO::Common::_set_dbd_inheritance($dbd, @_);
+sub _init {
+    my ($class, $dbo, $table) = @_;
+    (my $schema, $table, my $me) = $dbo->table_info($table) or croak 'No such table: '.$table;
+    bless { %$me, Schema => $schema, Name => $table, DBO => $dbo, LastInsertID => undef }, $class;
 }
 
 =head3 C<tables>
@@ -78,7 +76,7 @@ sub tables {
 
 sub _table_alias {
     return if $_[0] == $_[1];
-    ouch 'The table is not in this query';
+    croak 'The table is not in this query';
 }
 
 sub _quoted_name {
@@ -98,7 +96,7 @@ The C<**> method is a shortcut for the C<column> method.
 
 sub column {
     my ($me, $col) = @_;
-    ouch 'Invalid column '.$me->_qi($col).' in table '.$me->_quoted_name
+    croak 'Invalid column '.$me->_qi($col).' in table '.$me->_quoted_name
         unless exists $me->{Column_Idx}{$col};
     $me->{Column}{$col} ||= bless [$me, $col], 'DBIx::DBO::Column';
 }
@@ -116,8 +114,7 @@ Returns a new L<DBIx::DBO::Row|DBIx::DBO::Row> object for this table.
 
 sub row {
     my $me = shift;
-    my $row_class = $me->config('RowClass');
-    return $row_class ? $row_class->new($me->{DBO}, $me) : $me->{DBO}->row($me);
+    $me->_row_class->new($me->{DBO}, $me);
 }
 
 =head3 C<fetch_row>
@@ -204,8 +201,8 @@ the autogenerated ID (if there was one) from the last inserted row.
 
 sub insert {
     my $me = shift;
-    ouch 'Called insert() without args on table '.$me->_quoted_name unless @_;
-    ouch 'Wrong number of arguments' if @_ & 1;
+    croak 'Called insert() without args on table '.$me->_quoted_name unless @_;
+    croak 'Wrong number of arguments' if @_ & 1;
     my @cols;
     my @vals;
     my @bind;
